""" example SmartSPIM instrument """

import datetime

from aind_data_schema_models.organizations import Organization

from aind_data_schema.components.devices import Filter, Laser, MotorizedStage, OpticalTable, ScanningStage
from aind_data_schema.core.instrument import Com, Detector, Instrument, Objective

inst = Instrument(
    instrument_id="SmartSPIM2-1",
    instrument_type="SmartSPIM",
    manufacturer=Organization.LIFECANVAS,
    modification_date=datetime.date(2023, 10, 4),
    objectives=[
        Objective(
            name="TLX Objective 1",
            numerical_aperture=0.2,
            magnification=3.6,
            manufacturer=Organization.LIFECANVAS,
            immersion="multi",
            notes="Thorlabs TL4X-SAP with LifeCanvas dipping cap and correction optics.",
            serial_number="Unknown-1",
        ),
        Objective(
            name="TLX Objective 2",
            numerical_aperture=0.12,
            magnification=1.625,
            manufacturer=Organization.LIFECANVAS,
            immersion="multi",
            notes="Thorlabs TL2X-SAP with LifeCanvas dipping cap and correction optics.",
            serial_number="Unknown-2",
        ),
    ],
    detectors=[
        Detector(
            detector_type="Camera",
            data_interface="USB",
            name="Camera 1",
            cooling="Air",
            manufacturer=Organization.HAMAMATSU,
            model="C14440-20UP",
            serial_number="001284",
        ),
    ],
    light_sources=[
        Laser(
            name="Ex_445",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=445,
            maximum_power=200,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
        Laser(
            name="Ex_488",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=488,
            maximum_power=150,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
        Laser(
            name="Ex_561",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=561,
            maximum_power=150,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
        Laser(
            name="Ex_594",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=594,
            maximum_power=100,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
        Laser(
            name="Ex_639",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=639,
            maximum_power=160,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
        Laser(
            name="Ex_690",
            device_type="Laser",
            coupling="Single-mode fiber",
            wavelength=690,
            maximum_power=160,
            serial_number="VL08223M03",
            manufacturer=Organization.VORTRAN,
        ),
    ],
    fluorescence_filters=[
        Filter(
            name="Em_469",
            filter_type="Band pass",
            manufacturer=Organization.SEMROCK,
            diameter=25,
            thickness=2.0,
            model="FF01-469/35-25",
            filter_wheel_index=0,
            serial_number="Unknown-0",
        ),
        Filter(
            name="Em_525",
            filter_type="Band pass",
            manufacturer=Organization.SEMROCK,
            diameter=25,
            thickness=2.0,
            model="FF01-525/45-25",
            filter_wheel_index=1,
            serial_number="Unknown-1",
        ),
        Filter(
            name="Em_593",
            filter_type="Band pass",
            manufacturer=Organization.SEMROCK,
            diameter=25,
            thickness=2.0,
            model="FF01-593/40-25",
            filter_wheel_index=2,
            serial_number="Unknown-2",
        ),
        Filter(
            name="Em_624",
            filter_type="Band pass",
            manufacturer=Organization.SEMROCK,
            diameter=25,
            thickness=2.0,
            model="FF01-624/40-25",
            filter_wheel_index=3,
            serial_number="Unknown-3",
        ),
        Filter(
            name="Em_667",
            filter_type="Band pass",
            manufacturer=Organization.CHROMA,
            diameter=25,
            thickness=2.0,
            model="ET667/30m",
            filter_wheel_index=4,
            serial_number="Unknown-4",
        ),
        Filter(
            name="Em_700",
            filter_type="Long pass",
            manufacturer=Organization.THORLABS,
            diameter=25,
            thickness=2.0,
            model="FELH0700",
            filter_wheel_index=5,
            serial_number="Unknown-5",
        ),
    ],
    motorized_stages=[
        MotorizedStage(
            model="LS-100",
            manufacturer=Organization.ASI,
            serial_number="Unknown-1",
            travel=100,
            name="Focus stage",
        ),
        MotorizedStage(
            model="L12-20F-4",
            manufacturer=Organization.MIGHTY_ZAP,
            serial_number="Unknown-5",
            travel=41,
            name="Cylindrical lens #1",
        ),
        MotorizedStage(
            model="L12-20F-4",
            manufacturer=Organization.MIGHTY_ZAP,
            serial_number="Unknown-6",
            travel=41,
            name="Cylindrical lens #2",
        ),
        MotorizedStage(
            model="L12-20F-4",
            manufacturer=Organization.MIGHTY_ZAP,
            serial_number="Unknown-7",
            travel=41,
            name="Cylindrical lens #3",
        ),
        MotorizedStage(
            model="L12-20F-4",
            manufacturer=Organization.MIGHTY_ZAP,
            serial_number="Unknown-8",
            travel=41,
            name="Cylindrical lens #4",
        ),
    ],
    scanning_stages=[
        ScanningStage(
            model="LS-50",
            manufacturer=Organization.ASI,
            serial_number="Unknown-2",
            stage_axis_direction="Detection axis",
            stage_axis_name="Z",
            travel=50,
            name="Sample stage Z",
        ),
        ScanningStage(
            model="LS-50",
            manufacturer=Organization.ASI,
            serial_number="Unknown-3",
            stage_axis_direction="Illumination axis",
            stage_axis_name="X",
            travel=50,
            name="Sample stage X",
        ),
        ScanningStage(
            model="LS-50",
            manufacturer=Organization.ASI,
            serial_number="Unknown-4",
            stage_axis_direction="Perpendicular axis",
            stage_axis_name="Y",
            travel=50,
            name="Sample stage Y",
        ),
    ],
    optical_tables=[
        OpticalTable(
            name="Main optical table",
            length=36,
            width=48,
            vibration_control=False,
            model="VIS2424-IG2-125A",
            manufacturer=Organization.MKS_NEWPORT,
            serial_number="Unknown",
        )
    ],
    com_ports=[
        Com(hardware_name="Laser Launch", com_port="COM4"),
        Com(
            hardware_name="ASI Tiger",
            com_port="COM3",
        ),
        Com(
            hardware_name="MightyZap",
            com_port="COM9",
        ),
    ],
    humidity_control=False,
    temperature_control=False,
)

serialized = inst.model_dump_json()
deserialized = Instrument.model_validate_json(serialized)
deserialized.write_standard_file(prefix="aind_smartspim")
